@@ -1,6 +1,6 @@
 """Arbitrary precision arithmetic."""
 
-__version__ = '5.1.0'
+__version__ = '5.2.0'
 
 try:
     from ._rithm import (Fraction,
@@ -258,6 +258,23 @@ except ImportError:
         def __float__(self) -> float:
             return self._numerator._value / self._denominator._value
 
+        @_overload
+        def __floordiv__(self, other: _Any) -> _Any:
+            ...
+
+        @_overload
+        def __floordiv__(self, other: _Union['Fraction', Int]) -> Int:
+            ...
+
+        def __floordiv__(self, other: _Union['Fraction', Int, _Any]
+                         ) -> _Union[Int, _Any]:
+            return ((self.numerator * other.denominator)
+                    // (self.denominator * other.numerator)
+                    if isinstance(other, Fraction)
+                    else (self.numerator // (self.denominator * other)
+                          if isinstance(other, Int)
+                          else NotImplemented))
+
         def __ge__(self, other: 'Fraction') -> bool:
             return (self.numerator * other.denominator
                     >= other.numerator * self.denominator
@@ -293,6 +310,25 @@ except ImportError:
                           if isinstance(other, Int)
                           else NotImplemented))
 
+        @_overload
+        def __mod__(self, other: _Any) -> _Any:
+            ...
+
+        @_overload
+        def __mod__(self, other: _Union['Fraction', Int]) -> 'Fraction':
+            ...
+
+        def __mod__(self, other: _Union['Fraction', Int, _Any]
+                    ) -> _Union['Fraction', _Any]:
+            return (Fraction((self.numerator * other.denominator)
+                             % (self.denominator * other.numerator),
+                             self.denominator * other.denominator)
+                    if isinstance(other, Fraction)
+                    else (Fraction(self.numerator % (self.denominator * other),
+                                   self.denominator)
+                          if isinstance(other, Int)
+                          else NotImplemented))
+
         @_overload
         def __mul__(self, other: _Any) -> _Any:
             ...
@@ -343,6 +379,34 @@ except ImportError:
                     if isinstance(other, Int)
                     else NotImplemented)
 
+        @_overload
+        def __rfloordiv__(self, other: _Any) -> _Any:
+            ...
+
+        @_overload
+        def __rfloordiv__(self, other: Int) -> Int:
+            ...
+
+        def __rfloordiv__(self, other: _Union[Int, _Any]) -> _Union[Int, _Any]:
+            return ((other * self.denominator) // self.numerator
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
+        @_overload
+        def __rmod__(self, other: _Any) -> _Any:
+            ...
+
+        @_overload
+        def __rmod__(self, other: Int) -> 'Fraction':
+            ...
+
+        def __rmod__(self, other: _Union[Int, _Any]
+                     ) -> _Union['Fraction', _Any]:
+            return (Fraction((other * self.denominator) % self.numerator,
+                             self.denominator)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         @_overload
         def __rmul__(self, other: _Any) -> _Any:
             ...
