@@ -5,13 +5,14 @@ use std::ops::{Add, AddAssign, Div, Mul, MulAssign, Neg, Sub, SubAssign};
 
 use crate::big_int::BigInt;
 use crate::digits::{
-    AdditiveDigit, DigitConvertibleFromFloat, GcdDigit, LeftShiftableDigit, MultiplicativeDigit,
-    UnitaryDigit,
+    AdditiveDigit, DigitConvertibleFromFloat, EuclidDivisibleDigit, GcdDigit, LeftShiftableDigit,
+    MultiplicativeDigit, UnitaryDigit,
 };
 use crate::traits::{
-    Abs, AdditiveMonoid, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedPow, CheckedShl,
-    DivisivePartialMagma, Float, GcdMagma, Maybe, ModularUnaryAlgebra, MultiplicativeMonoid,
-    NegatableUnaryAlgebra, Oppositive, Pow, SubtractiveMagma, Unitary, Zeroable,
+    Abs, AdditiveMonoid, CheckedDiv, CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid,
+    CheckedPow, CheckedRemEuclid, CheckedShl, DivisivePartialMagma, Float, GcdMagma, Maybe,
+    ModularUnaryAlgebra, MultiplicativeMonoid, NegatableUnaryAlgebra, Oppositive, Pow,
+    SubtractiveMagma, Unitary, Zeroable,
 };
 use crate::utils;
 
@@ -265,6 +266,74 @@ impl<Digit: Eq + GcdDigit + MultiplicativeDigit, const SEPARATOR: char, const SH
     }
 }
 
+impl<
+        Component: Clone + CheckedDivEuclid<Output = Option<Component>> + Eq + MultiplicativeMonoid + Zeroable,
+    > CheckedDivEuclid for Fraction<Component>
+{
+    type Output = Option<Component>;
+
+    fn checked_div_euclid(self, divisor: Self) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            (self.numerator * divisor.denominator)
+                .checked_div_euclid(divisor.numerator * self.denominator)
+        }
+    }
+}
+
+impl<
+        Component: Clone + CheckedDivEuclid<Output = Option<Component>> + Eq + MultiplicativeMonoid + Zeroable,
+    > CheckedDivEuclid<Component> for Fraction<Component>
+{
+    type Output = Option<Component>;
+
+    fn checked_div_euclid(self, divisor: Component) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            self.numerator
+                .checked_div_euclid(divisor * self.denominator)
+        }
+    }
+}
+
+macro_rules! plain_checked_div_euclid_fraction_impl {
+    ($($t:ty)*) => ($(
+    impl CheckedDivEuclid<Fraction<Self>> for $t
+    {
+        type Output = Option<Self>;
+
+        fn checked_div_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+            if divisor.is_zero() {
+                None
+            } else {
+                (self * divisor.denominator).checked_div_euclid(divisor.numerator)
+            }
+        }
+    }
+    )*)
+}
+
+plain_checked_div_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+impl<
+        Digit: Eq + EuclidDivisibleDigit + MultiplicativeDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > CheckedDivEuclid<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Option<Self>;
+
+    fn checked_div_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            (self * divisor.denominator).checked_div_euclid(divisor.numerator)
+        }
+    }
+}
+
 impl<
         Component: Clone
             + Eq
@@ -300,6 +369,115 @@ impl<
     }
 }
 
+impl<
+        Component: Clone
+            + CheckedRemEuclid<Output = Option<Component>>
+            + DivisivePartialMagma
+            + Eq
+            + GcdMagma
+            + MultiplicativeMonoid
+            + Zeroable,
+    > CheckedRemEuclid for Fraction<Component>
+{
+    type Output = Option<Self>;
+
+    fn checked_rem_euclid(self, divisor: Self) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            let (numerator, denominator) = normalize_components_moduli(
+                (self.numerator * divisor.denominator.clone())
+                    .checked_rem_euclid(divisor.numerator * self.denominator.clone())?,
+                self.denominator * divisor.denominator,
+            );
+            Some(Self {
+                numerator,
+                denominator,
+            })
+        }
+    }
+}
+
+impl<
+        Component: Clone
+            + CheckedRemEuclid<Output = Option<Component>>
+            + DivisivePartialMagma
+            + Eq
+            + GcdMagma
+            + MultiplicativeMonoid
+            + Zeroable,
+    > CheckedRemEuclid<Component> for Fraction<Component>
+{
+    type Output = Option<Self>;
+
+    fn checked_rem_euclid(self, divisor: Component) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            let (numerator, denominator) = normalize_components_moduli(
+                self.numerator
+                    .checked_rem_euclid(divisor * self.denominator.clone())?,
+                self.denominator,
+            );
+            Some(Self {
+                numerator,
+                denominator,
+            })
+        }
+    }
+}
+
+macro_rules! plain_checked_rem_euclid_fraction_impl {
+    ($($t:ty)*) => ($(
+        impl CheckedRemEuclid<Fraction<Self>> for $t
+        {
+            type Output = Option<Fraction<Self>>;
+
+            fn checked_rem_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+                if divisor.is_zero() {
+                    None
+                } else {
+                    let (numerator, denominator) = normalize_components_moduli(
+                        (self * divisor.denominator)
+                            .checked_rem_euclid(divisor.numerator)?,
+                        divisor.denominator,
+                    );
+                    Some(Fraction::<Self> {
+                        numerator,
+                        denominator,
+                    })
+                }
+            }
+        }
+        )*)
+}
+
+plain_checked_rem_euclid_fraction_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
+impl<
+        Digit: Eq + EuclidDivisibleDigit + GcdDigit + MultiplicativeDigit,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > CheckedRemEuclid<Fraction<Self>> for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Option<Fraction<Self>>;
+
+    fn checked_rem_euclid(self, divisor: Fraction<Self>) -> Self::Output {
+        if divisor.is_zero() {
+            None
+        } else {
+            let (numerator, denominator) = normalize_components_moduli(
+                (self * divisor.denominator.clone()).checked_rem_euclid(divisor.numerator)?,
+                divisor.denominator,
+            );
+            Some(Fraction::<Self> {
+                numerator,
+                denominator,
+            })
+        }
+    }
+}
+
 impl<Component: Clone + Display + Eq + Unitary> Display for Fraction<Component> {
     fn fmt(&self, formatter: &mut Formatter) -> std::fmt::Result {
         if self.denominator.is_one() {
