@@ -278,8 +278,9 @@ impl PyNumberProtocol for PyInt {
             .checked_shl(rhs.0)
             .map(PyInt)
             .map_err(|reason| match reason {
-                big_int::ShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
-                _ => PyValueError::new_err(reason.to_string()),
+                big_int::LeftShiftError::NegativeShift => PyValueError::new_err(reason.to_string()),
+                big_int::LeftShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
+                big_int::LeftShiftError::TooLarge => PyOverflowError::new_err(reason.to_string()),
             })
     }
 
@@ -341,8 +342,9 @@ impl PyNumberProtocol for PyInt {
             .checked_shr(rhs.0)
             .map(PyInt)
             .map_err(|reason| match reason {
-                big_int::ShiftError::OutOfMemory => PyMemoryError::new_err(reason.to_string()),
-                _ => PyValueError::new_err(reason.to_string()),
+                big_int::RightShiftError::NegativeShift => {
+                    PyValueError::new_err(reason.to_string())
+                }
             })
     }
 
@@ -443,6 +445,48 @@ impl PyNumberProtocol for PyFraction {
         }
     }
 
+    fn __floordiv__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
+        let py = rhs.py();
+        if rhs.is_instance::<PyFraction>()? {
+            match lhs.0.checked_div_euclid(rhs.extract::<PyFraction>()?.0) {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else if rhs.is_instance::<PyInt>()? {
+            match lhs.0.checked_div_euclid(rhs.extract::<PyInt>()?.0) {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __mod__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
+        let py = rhs.py();
+        if rhs.is_instance::<PyFraction>()? {
+            match lhs.0.checked_rem_euclid(rhs.extract::<PyFraction>()?.0) {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else if rhs.is_instance::<PyInt>()? {
+            match lhs.0.checked_rem_euclid(rhs.extract::<PyInt>()?.0) {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
     fn __mul__(lhs: PyFraction, rhs: &PyAny) -> PyResult<PyObject> {
         let py = rhs.py();
         if rhs.is_instance::<PyFraction>()? {
@@ -480,6 +524,42 @@ impl PyNumberProtocol for PyFraction {
         }
     }
 
+    fn __rfloordiv__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance::<PyInt>()? {
+            match other
+                .extract::<PyInt>()?
+                .0
+                .checked_div_euclid(self.0.clone())
+            {
+                Some(value) => Ok(PyInt(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
+    fn __rmod__(&self, other: &PyAny) -> PyResult<PyObject> {
+        let py = other.py();
+        if other.is_instance::<PyInt>()? {
+            match other
+                .extract::<PyInt>()?
+                .0
+                .checked_rem_euclid(self.0.clone())
+            {
+                Some(value) => Ok(PyFraction(value).into_py(py)),
+                None => Err(PyZeroDivisionError::new_err(
+                    UNDEFINED_DIVISION_ERROR_MESSAGE,
+                )),
+            }
+        } else {
+            Ok(py.NotImplemented())
+        }
+    }
+
     fn __rmul__(&self, other: &PyAny) -> PyResult<PyObject> {
         let py = other.py();
         if other.is_instance::<PyInt>()? {
